unit EnvelopeFormUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls, SMDUtils;

type

  { TFormEnvelopes }

  TFormEnvelopes = class(TForm)
    ButtonSet: TButton;
    ButtonNewEnvelope: TButton;
    ButtonDeleteEnvelope: TButton;
    ButtonClose: TButton;
    EditBefore: TEdit;
    EditAfter: TEdit;
    GroupBoxEnvelope: TGroupBox;
    GroupBoxStrings: TGroupBox;
    LabelEllipsis: TLabel;
    LabelBefore: TLabel;
    LabelAfter: TLabel;
    ListBoxEnvelope: TListBox;
    procedure ButtonCloseClick(Sender: TObject);
    procedure ButtonDeleteEnvelopeClick(Sender: TObject);
    procedure ButtonNewEnvelopeClick(Sender: TObject);
    procedure ButtonSetClick(Sender: TObject);
    procedure EditAfterChange(Sender: TObject);
    procedure EditBeforeChange(Sender: TObject);
    procedure ListBoxEnvelopeSelectionChange(Sender: TObject; User: boolean);

  private
    FCurrent : PEnvelope;
    procedure CheckEnvelopeStrings;

  public
    { public declarations }
  end;

var
  FormEnvelopes: TFormEnvelopes;

implementation

{$R *.lfm}

const
  BUTTONSET_CAPTION : array[Boolean] of string = ('Check input', 'Set');

{ TFormEnvelopes }

procedure TFormEnvelopes.ButtonCloseClick(Sender: TObject);
begin
  EnvelopeManager.SaveToFile;
  ModalResult := mrOK;
end;

procedure TFormEnvelopes.ButtonDeleteEnvelopeClick(Sender: TObject);
begin

  EnvelopeManager.Delete(ListBoxEnvelope.Items[ListBoxEnvelope.ItemIndex]);
  EnvelopeManager.CopyTo(ListBoxEnvelope.Items, False);
  ListBoxEnvelopeSelectionChange(Sender, False);

end;

procedure TFormEnvelopes.ButtonNewEnvelopeClick(Sender: TObject);
var
  N : string;

begin

  N := Trim(InputBox('SMD - New Envelope', 'Enter envelope name:', ''));
  if (N = '') OR Assigned(EnvelopeManager.Find(N)) then
   begin
     ShowMessage('Error: Envelope name empty or already exists.');
     Exit;
   end;

  EnvelopeManager.Add(N, '', '');
  EnvelopeManager.CopyTo(ListBoxEnvelope.Items, False);
  ListBoxEnvelope.ItemIndex := ListBoxEnvelope.Items.IndexOf(N);

end;

procedure TFormEnvelopes.ButtonSetClick(Sender: TObject);
begin

  FCurrent^.Before := EditBefore.Text;
  FCurrent^.After := EditAfter.Text;
  EnvelopeManager.SaveToFile;
  // Until next change...
  ButtonSet.Enabled := False;

end;

procedure TFormEnvelopes.EditAfterChange(Sender: TObject);
begin
  CheckEnvelopeStrings;
end;

procedure TFormEnvelopes.EditBeforeChange(Sender: TObject);
begin
  CheckEnvelopeStrings;
end;

procedure TFormEnvelopes.ListBoxEnvelopeSelectionChange(Sender: TObject;
  User: boolean);
var
  ActualSelection : Boolean;

begin

  ActualSelection := ListBoxEnvelope.ItemIndex >= 0;
  if ActualSelection then
   begin

     FCurrent := EnvelopeManager.Find(ListBoxEnvelope.Items[ListBoxEnvelope.ItemIndex]);
     EditBefore.Text := FCurrent^.Before;
     EditAfter.Text := FCurrent^.After;

   end else
    begin

      FCurrent := nil;
      EditBefore.Text := '';
      EditAfter.Text := '';

    end;

  ButtonDeleteEnvelope.Enabled := ActualSelection;
  GroupBoxStrings.Enabled := ActualSelection;
  // Nothing changed yet
  ButtonSet.Enabled := False;

end;

procedure TFormEnvelopes.CheckEnvelopeStrings;
var
  Valid : Boolean;
  s     : string = '';

begin

  Valid := EscapedStrToRaw(EditBefore.Text, s, True) AND
           EscapedStrToRaw(EditAfter.Text, s, True);
  ButtonSet.Enabled := Valid;
  ButtonSet.Caption := BUTTONSET_CAPTION[Valid];

end;

end.

