unit MainFormUnit;

{$mode objfpc}{$H+}

interface

uses
  Forms, ExtCtrls, StdCtrls, SMDUtils, COMPort, Classes, Graphics, Controls,
  BaudFormUnit, EnvelopeFormUnit, Dialogs, SdpoSerial, SysUtils, Clipbrd;

resourcestring
  SMD_ABOUT = 'SMD - Serial Monitor Deluxe' + #13 +
              'Version 0.1' + #13 +
              'Written in Object Pascal (FPC/Lazarus)' + #13 +
              'By Ido Gendel, 2014' + #13 +
              'email: info@idogendel.com' + #13 +
              'No guarantees, no warranties. Free to use. Enjoy!';

type

  { TFormMain }

  TFormMain = class(TForm)
    ButtonSend: TButton;
    ButtonClear: TButton;
    ButtonAbout: TButton;
    ButtonCopy: TButton;
    ButtonPortReset: TButton;
    CheckBoxNonPrintables: TCheckBox;
    CheckBoxAutoscroll: TCheckBox;
    ComboBoxSend: TComboBox;
    ComboBoxEnvelope: TComboBox;
    ComboBoxBaud: TComboBox;
    ComboBoxPort: TComboBox;
    LabelEnvelope: TLabel;
    LabelBaud: TLabel;
    LabelPort: TLabel;
    ListBoxData: TListBox;
    Panel1: TPanel;
    PanelReceive: TPanel;
    PanelSend: TPanel;
    PanelPort: TPanel;
    PortCheckTimer: TTimer;
    procedure ButtonAboutClick(Sender: TObject);
    procedure ButtonClearClick(Sender: TObject);
    procedure ButtonCopyClick(Sender: TObject);
    procedure ButtonPortResetClick(Sender: TObject);
    procedure ButtonSendClick(Sender: TObject);
    procedure CheckBoxAutoscrollChange(Sender: TObject);
    procedure CheckBoxNonPrintablesChange(Sender: TObject);
    procedure ComboBoxBaudChange(Sender: TObject);
    procedure ComboBoxEnvelopeChange(Sender: TObject);
    procedure ComboBoxPortSelect(Sender: TObject);
    procedure ComboBoxSendChange(Sender: TObject);
    procedure ComboBoxSendKeyPress(Sender: TObject; var Key: char);
    procedure ComboBoxSendSelect(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PortCheckTimerTimer(Sender: TObject);

    procedure ProcessIncomingData(s : string);
    procedure GetRx(Sender : TObject);

  private
    FPosition : TRect;
    FLastValidBaudrate : string;
    FCurrentEnvelope : PEnvelope;
    FIncoming : TStringList;
    FAutoscroll : Boolean;

    procedure LoadPosition;
    procedure SavePosition;
    procedure PopulatePorts;
    procedure PopulateSendHistory;
    procedure PopulateBaudrates;
    procedure PopulateEnvelopes(Selected : string = '');
    procedure GetCurrentEnvelopeStrs;
    function AddDataItem(const s : string; const ExtraLine : Boolean = False) : Integer;

  public
  end;

const
  NEW_LINE_MARK       = #10;
  BUTTONSEND_CAPTION  : array[Boolean] of string = ('Check input', 'Send');
  NONPRINTABLES_OP    : array[Boolean] of string = ('OFF', 'ON (except #13 and #10)');
  NONPRINTABLES_MSG   = '  // Non-printable character display ';
  ERASE_HISTORY_STR   = '  (Clear All)';
  BAUD_STANDARD_STR   = '--- Standard';
  BAUD_CUSTOM_STR     = '--- Custom';
  BAUD_EDIT_STR       = '  (Edit...)';
  ENVELOPE_EDIT_STR   = '  (Edit...)';
  SEND_ECHO_STR       = '  // Sent: ';
  RESET_NOTIFY_STR    = '  // Reset';
  PORT_NOTIFY_STR     = '  // Port Change';
  CLIPBOARD_NEW_LINE  = #13 + #10;
  FORMPOS_FILENAME    = 'Position.dat';

var
  FormMain : TFormMain;

implementation

{$R *.lfm}

{ TFormMain }

function TFormMain.AddDataItem(const s : string; const ExtraLine : Boolean = False) : Integer;
begin

  try
    Result := ListBoxData.Items.Add(s);
    if ExtraLine then ListBoxData.Items.Add('');

  except
    OutOfMemoryError;
  end;

  if FAutoscroll then
   begin
     ListBoxData.ItemIndex := ListBoxData.Count - 1;
     ListBoxData.Selected[ListBoxData.ItemIndex] := False;
   end;

end;

procedure TFormMain.FormCreate(Sender: TObject);
begin

  LoadPosition;
  PopulatePorts;
  PopulateSendHistory;
  PopulateBaudrates;
  ComboBoxBaud.ItemIndex := ComboBoxBaud.Items.IndexOf(StartupBaudStr);
  ComboBoxBaudChange(Sender); // Apply change
  PopulateEnvelopes;
  FAutoscroll := CheckBoxAutoscroll.Checked;
  // Set initial state of Send button
  GetCurrentEnvelopeStrs;
  ComboBoxSendChange(Sender);

  FIncoming := TStringList.Create;
  FIncoming.Delimiter := #13;
  Serial.AssignRxHandler(@GetRx);

end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  FIncoming.Free;
end;

procedure TFormMain.PortCheckTimerTimer(Sender: TObject);
begin

  if Serial.AvailablePortsChanged then
   begin
     PanelPort.Color := clHighlight;
     PopulatePorts;
   end else
     PanelPort.Color := clDefault;

end;

procedure TFormMain.ComboBoxSendChange(Sender: TObject);
var
  Processed : string = '';

begin
  ButtonSend.Enabled := EscapedStrToRaw(ComboBoxSend.Text, Processed);
  ButtonSend.Caption := BUTTONSEND_CAPTION[ButtonSend.Enabled];
end;

procedure TFormMain.ComboBoxSendKeyPress(Sender: TObject; var Key: char);
begin
  if (Key = #13) AND (ButtonSend.Enabled) then ButtonSendClick(Sender)
   else ComboBoxSendChange(Sender);
end;

procedure TFormMain.ComboBoxSendSelect(Sender: TObject);
begin

  if ComboBoxSend.Items[ComboBoxSend.ItemIndex] = ERASE_HISTORY_STR then
   begin

     SendHistory.Clear;
     //ComboBoxSend.Items.Clear;
     ComboBoxSend.Clear;
     ComboBoxSendChange(Sender);

   end;

end;

procedure TFormMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  SavePosition;
end;

procedure TFormMain.ButtonPortResetClick(Sender: TObject);
begin
  Serial.ResetDTR;
  AddDataItem(RESET_NOTIFY_STR, True);
end;

procedure TFormMain.ButtonAboutClick(Sender: TObject);
begin
  ShowMessage(SMD_ABOUT);
end;

procedure TFormMain.ButtonClearClick(Sender: TObject);
begin
  ListBoxData.Clear;
  CheckBoxNonPrintablesChange(Sender);
end;

procedure TFormMain.ButtonCopyClick(Sender: TObject);
var
  s : string;
  i : Integer;

begin

  s := '';
  try
    for i := 0 to ListBoxData.Items.Count - 1 do
     s := s + ListBoxData.Items[i] + CLIPBOARD_NEW_LINE;
  except
    OutOfMemoryError;
  end;
  Clipboard.AsText := s;

end;

procedure TFormMain.ButtonSendClick(Sender: TObject);
var
  Pre       : string;
  Processed : string = '';

begin

  // Input is legit, otherwise button could not be pressed
  SendHistory.Add(ComboBoxSend.Text);
  PopulateSendHistory;
  Pre := FCurrentEnvelope^.Before + ComboBoxSend.Text + FCurrentEnvelope^.After;
  EscapedStrToRaw(Pre, Processed);
  Serial.Send(Processed);
  AddDataItem(SEND_ECHO_STR + Pre, True);

end;

procedure TFormMain.CheckBoxAutoscrollChange(Sender: TObject);
begin
  FAutoscroll := CheckBoxAutoscroll.Checked;
end;

procedure TFormMain.CheckBoxNonPrintablesChange(Sender: TObject);
begin
  AddDataItem(NONPRINTABLES_MSG + NONPRINTABLES_OP[CheckBoxNonPrintables.Checked], True);
end;

procedure TFormMain.ComboBoxBaudChange(Sender: TObject);
var
  temp : Integer;

begin

  if ComboBoxBaud.Items[ComboBoxBaud.ItemIndex] = BAUD_EDIT_STR then
   begin

     FormBaud.EditBaudrates.Text := BaudManager.CustomBaudsAsCommaText;
     FormBaud.LabelRange.Caption := IntToStr(BAUD_ABSOLUTE_MIN) + ' to ' + IntToStr(BAUD_ABSOLUTE_MAX);
     if FormBaud.ShowModal = mrOK then
      begin
        BaudManager.CopyCustomFrom(UserInputBaudrates);
        PopulateBaudrates;
      end;

   end;

   // Restore a valid baudrate after text selections
   try

     temp := StrToInt(ComboBoxBaud.Items[ComboBoxBaud.ItemIndex]);
     FLastValidBaudrate := ComboBoxBaud.Items[ComboBoxBaud.ItemIndex];
     Serial.BaudRate := StrToInt(FLastValidBaudrate);

   except

     temp := ComboBoxBaud.Items.IndexOf(FLastValidBaudrate);
     if temp >= 0 then ComboBoxBaud.ItemIndex := temp
      else ComboBoxBaud.ItemIndex := ComboBoxBaud.Items.IndexOf(DEFAULT_BAUDRATE_STR);
     ComboBoxBaudChange(Sender); // Apply change

   end;

  StartupBaudStr := ComboBoxBaud.Items[ComboBoxBaud.ItemIndex];

end;

procedure TFormMain.ComboBoxEnvelopeChange(Sender: TObject);
var
  i : Integer;
  s : string;

begin

  if ComboBoxEnvelope.Text = ENVELOPE_EDIT_STR then
   begin

     EnvelopeManager.CopyTo(FormEnvelopes.ListBoxEnvelope.Items, False);
     FormEnvelopes.ListBoxEnvelope.ItemIndex := -1;
     FormEnvelopes.ListBoxEnvelopeSelectionChange(Sender, False);
     FormEnvelopes.ShowModal;

     i := FormEnvelopes.ListBoxEnvelope.ItemIndex;
     if i < 0 then s := ''
      else s := FormEnvelopes.ListBoxEnvelope.Items[i];

     PopulateEnvelopes(s);

   end else
    begin
      GetCurrentEnvelopeStrs;
    end;

end;

procedure TFormMain.ComboBoxPortSelect(Sender: TObject);
begin
  Serial.Port := ComboBoxPort.Text;
  AddDataItem(PORT_NOTIFY_STR, True);
end;

procedure TFormMain.ProcessIncomingData(s : string);
var
  i, p : Integer;
  temp : string;

begin

  FIncoming.Clear;
  s := s + NEW_LINE_MARK;
  p := Pos(NEW_LINE_MARK, s);
  while p > 0 do
   begin

     temp := Copy(s, 1, p - 1);
     if CheckBoxNonPrintables.Checked then temp := StrToEscaped(temp);
     FIncoming.Add(temp);
     Delete(s, 1, p);
     p := Pos(NEW_LINE_MARK, s);

   end;

  if ListBoxData.Items.Count = 0 then
    AddDataItem('');

  ListBoxData.Items[ListBoxData.Items.Count - 1] := ListBoxData.Items[ListBoxData.Items.Count - 1] + FIncoming[0];

  for i := 1 to FIncoming.Count - 1 do
   AddDataItem(FIncoming[i]);

end;

procedure TFormMain.GetRx(Sender : TObject);
begin
  ProcessIncomingData(Serial.Receive);
end;

procedure TFormMain.LoadPosition;
var
  f : file of TRect;

begin

  AssignFile(f, FORMPOS_FILENAME);
  try

    Reset(f);
    Read(f, FPosition);
    CloseFile(f);

    Position := poDesigned;
    Top := FPosition.Top;
    Height := FPosition.Bottom;
    Left := FPosition.Left;
    Width := FPosition.Right;

  except
  end;

end;

procedure TFormMain.SavePosition;
var
  f : file of TRect;

begin

  FPosition.Top := Top;
  FPosition.Bottom := Height;
  FPosition.Left := Left;
  FPosition.Right := Width;

  AssignFile(f, FORMPOS_FILENAME);
  try

    Rewrite(f);
    Write(f, FPosition);
    CloseFile(f);

  except
    raise Exception.Create(ErrSaveToFile + ' ("' + FORMPOS_FILENAME + '")');
  end;

end;

procedure TFormMain.PopulatePorts;
var
  i : Integer;

begin

  ComboBoxPort.Items.Assign(Serial.AvailablePorts);
  ComboBoxPort.Items.Insert(0, NO_PORT_STR);
  i := ComboBoxPort.Items.IndexOf(Serial.Port);
  if i >= 0 then ComboBoxPort.ItemIndex := i
   else ComboBoxPort.ItemIndex := 0;

end;

procedure TFormMain.PopulateSendHistory;
begin
  SendHistory.CopyTo(ComboBoxSend.Items);
  if ComboBoxSend.Items.Count > 0 then ComboBoxSend.Items.Add(ERASE_HISTORY_STR);
end;

procedure TFormMain.PopulateBaudrates;
begin

  BaudManager.CopyTo(ComboBoxBaud.Items);
  ComboBoxBaud.Items.Insert(BaudManager.StdBaudratesCount, BAUD_CUSTOM_STR);
  ComboBoxBaud.Items.Insert(0, BAUD_STANDARD_STR);
  ComboBoxBaud.Items.Add(BAUD_EDIT_STR);

end;

procedure TFormMain.PopulateEnvelopes(Selected : string = '');
begin

  EnvelopeManager.CopyTo(ComboBoxEnvelope.Items);
  ComboBoxEnvelope.Items.Add(ENVELOPE_EDIT_STR);
  if Selected = '' then
    ComboBoxEnvelope.ItemIndex := 0 // Default
   else ComboBoxEnvelope.ItemIndex := ComboBoxEnvelope.Items.IndexOf(Selected);
  GetCurrentEnvelopeStrs;

end;

procedure TFormMain.GetCurrentEnvelopeStrs;
begin
  FCurrentEnvelope := EnvelopeManager.Find(ComboBoxEnvelope.Text);
end;

initialization

finalization

end.

